#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    3
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT  2
#define VENDOR_RQ_RUN_LIGHT  0

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

static uint16_t n;
static unsigned int xx;
uint8_t in_d(uint8_t pin)
{
    // read logic level of port-D's pin
    DDRD &= ~(1<<pin);
    return (PIND>>pin) & 1;
} 

void out_d(uint8_t pin, uint8_t val)
{
    // write logic level to port-D's pin
    DDRD |= 1<<pin;
    if (val)
        PORTD |= 1<<pin;
    else
        PORTD &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
	ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
	ADCSRA = 0b11000110;
	//         ||   -+-
	//         ||    |
	//         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
	//         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
	//         |
	//         +-> Enable ADC (บิต ADEN)
    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */
	static uint16_t x;
	//static unsigned int xx;
    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_d(led_no+5, !led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_d(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
	//Homework
	else if(rq->bRequest == VENDOR_RQ_GET_LIGHT)
	{
		x = read_adc(2);
		if(x<300){
		DDRC = 0b00111011;DDRB = 0b00000000;
	        switch(n%10)
			     {
			                     case 8: PORTC = 0b00010001; break;                                                              case 7: PORTC = 0b00110000; break;
			                     case 6: PORTC = 0b00110001; break;
				                 case 5: PORTC = 0b00011000; break;
				                 case 4: PORTC = 0b00011001; break;
				                 case 3: PORTC = 0b00111000; break;
			                     case 2: PORTC = 0b00111001; break;
			                     case 1: PORTC = 0b00010010; break;
			                     case 0: PORTC = 0b00010011; break;
 	 	 		                default : PORTC = 0b00010000;
		        }
		    _delay_ms(1);
		    DDRC = 0b00101011;DDRB = 0b00000010;
			switch(n/10)
			{
				case 4: PORTC = 0b00000001;PORTB = 0b00000010; break;
				case 3: PORTC = 0b00100000;PORTB = 0b00000010; break;
				case 2: PORTC = 0b00100001;PORTB = 0b00000010; break;
				case 1: PORTC = 0b00001000;PORTB = 0b00000010; break;
				case 0: PORTC = 0b00001001;PORTB = 0b00000010; break;
				case 9: PORTC = 0b00101000;PORTB = 0b00000010; break;
				case 8: PORTC = 0b00101001;PORTB = 0b00000010; break;
				case 7: PORTC = 0b00000010;PORTB = 0b00000010; break;
				case 6: PORTC = 0b00000011;PORTB = 0b00000010; break;
				default : PORTB = 0b00000010;PORTC = 0b00000000;
			}
			_delay_ms(1);
			//n=xx>>7;
			xx++;
			if(xx%100==0)n++;
			if (n>=60) n=0;//n = xx = 40;
			//xx++;
			DDRB = 0b00000010;
			PORTB = 0b00000000;
		x=n;
		}
		else n=0;
		usbMsgPtr = &x;
		return 2;//size in byte
	}
	else if(rq->bRequest == VENDOR_RQ_RUN_LIGHT && x<500)
	{
		   /*DDRC = 0b00111011;DDRB = 0b00000000;
	        switch(n%10)
			     {
			                     case 8: PORTC = 0b00010001; break;                                                              case 7: PORTC = 0b00110000; break;
			                     case 6: PORTC = 0b00110001; break;
				                 case 5: PORTC = 0b00011000; break;
				                 case 4: PORTC = 0b00011001; break;
				                 case 3: PORTC = 0b00111000; break;
			                     case 2: PORTC = 0b00111001; break;
			                     case 1: PORTC = 0b00010010; break;
			                     case 0: PORTC = 0b00010011; break;
 	 	 		                default : PORTC = 0b00010000;
		        }
		    _delay_ms(1);
		    DDRC = 0b00101011;DDRB = 0b00000010;
			switch(n/10)
			{
				case 4: PORTC = 0b00000001;PORTB = 0b00000010; break;
				case 3: PORTC = 0b00100000;PORTB = 0b00000010; break;
				case 2: PORTC = 0b00100001;PORTB = 0b00000010; break;
				case 1: PORTC = 0b00001000;PORTB = 0b00000010; break;
				case 0: PORTC = 0b00001001;PORTB = 0b00000010; break;
				case 9: PORTC = 0b00101000;PORTB = 0b00000010; break;
				case 8: PORTC = 0b00101001;PORTB = 0b00000010; break;
				case 7: PORTC = 0b00000010;PORTB = 0b00000010; break;
				case 6: PORTC = 0b00000011;PORTB = 0b00000010; break;
				default : PORTB = 0b00000010;PORTC = 0b00000000;
			}
			_delay_ms(1);
			//n=xx>>7;
			xx++;
			if(xx%10==0)n++;
			if (n>=60) n=x=0;//n = xx = 40;
			//xx++;
			DDRB = 0b00000010;
			PORTB = 0b00000000;*/
			usbMsgPtr = &x;
			return 2;
	}
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRD = (1<<PD5)|(1<<PD6)|(1<<PD7); // PD5..PD7: Output, PD0-PD4: Input

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
